import 'package:dio/dio.dart';
import 'package:base_repository/base_repository.dart';
import 'package:life_repository/entity/ipconfig_entity.dart';
import 'package:life_repository/entity/poi2_map_entity.dart';
import 'package:life_repository/entity/tip_map_entity.dart';
import '../amap_repository.dart';

class AMapRepositoryImpl extends AMapRepository {
  AMapRepositoryImpl.get();
  static final AMapRepositoryImpl _instance = AMapRepositoryImpl.get();
  factory AMapRepositoryImpl() {
    return _instance;
  }

  @override
  Future<List<MapTipEntity>> inputSearchTips({
    String? key,
    String? keywords,
    String? type,
    String? location,
    String? city,
    bool citylimit = false,
    String datatype = 'all',
    String? sig,
    String output = 'JSON',
  }) async {
    Options options = Options(headers: {'base_url': 'AMap'});
    Map<String, dynamic> parameters = {
      'key': key,
      'keywords': keywords,
      'citylimit': citylimit,
      'datatype': datatype,
      'output': output,
    };
    if (city != null) {
      parameters['city'] = city;
    }
    if (type != null) {
      parameters['type'] = type;
    }
    if (location != null) {
      parameters['location'] = location;
    }
    if (sig != null) {
      parameters['sig'] = sig;
    }
    dynamic result = await DSDioUtil.getInstance().get(
      '/v3/assistant/inputtips',
      options: options,
      queryParameters: parameters,
    );
    if (result['status'] == '1' && result['tips'] is List) {
      List<dynamic> tips = List<dynamic>.from(result['tips']);
      List<MapTipEntity> tipsEntity = List<MapTipEntity>.empty(growable: true);
      for (dynamic item in tips) {
        MapTipEntity entity = MapTipEntity(
          id: (item['id'] is String) ? (item['id'] as String?) : '',
          name: (item['name'] is String) ? (item['name'] as String?) : '',
          district:
              (item['district'] is String) ? (item['district'] as String?) : '',
          adcode: (item['adcode'] is String) ? (item['adcode'] as String?) : '',
          location:
              (item['location'] is String) ? (item['location'] as String?) : '',
          address:
              (item['address'] is String) ? (item['address'] as String?) : '',
          typecode:
              (item['typecode'] is String) ? (item['typecode'] as String?) : '',
          city: (item['city'] is List)
              ? (item['city'] as List?)
              : ['${item['city']}'],
        );
        if (entity.id!.isNotEmpty && entity.address!.isNotEmpty) {
          tipsEntity.add(entity);
        }
      }
      return tipsEntity;
    } else {
      throw SysnetError(
        code: -1,
        message: '未发现数据',
        type: SysnetErrorType.other,
        exception: null,
      );
    }
  }

  @override
  Future<List<Poi2MapEntity>> poi2AroundSearch({
    String? key,
    String? location,
    String? keywords,
    String? types,
    String radius = '3000',
    String sortrule = 'distance',
    String? region,
    String? showFields,
    int pageSize = 10,
    int pageNum = 1,
    String? sig,
    String output = 'json',
  }) async {
    Options options = Options(headers: {'base_url': 'AMap'});
    Map<String, dynamic> parameters = {
      'key': key,
      'location': location,
      'radius': radius,
      'sortrule': sortrule,
      'page_size': pageSize,
      'page_num': pageNum,
      'output': output,
    };
    if (keywords != null) {
      parameters['keywords'] = keywords;
    }
    if (types != null) {
      parameters['types'] = types;
    }
    if (region != null) {
      parameters['region'] = region;
    }
    if (showFields != null) {
      parameters['show_fields'] = showFields;
    }
    if (sig != null) {
      parameters['sig'] = sig;
    }
    parameters['output'] = output;
    dynamic result = await DSDioUtil.getInstance().get(
      '/v5/place/around',
      options: options,
      queryParameters: parameters,
    );
    List<Poi2MapEntity> poisEntity = List<Poi2MapEntity>.empty(growable:true);

    if (result['status'] == '1' && result['pois'] is List) {
      List<dynamic> pois = List<dynamic>.from(result['pois']);
      for (var item in pois) {
        poisEntity.add(Poi2MapEntity(
          name: item['name'],
          id: item['id'],
          location: item['location'],
          type: item['type'],
          typecode: item['typecode'],
          pname: item['pname'],
          cityname: item['cityname'],
          adname: item['adname'],
          address: item['address'],
          pcode: item['pcode'],
          citycode: item['citycode'],
          adcode: item['adcode'],
        ));
      }
    }
    return poisEntity;
  }

  @override
  Future<IpConfigEntity> ipConfigAddress({
    String? key,
    String? ip,
    String? sig,
  }) async {
    Options options = Options(headers: {'base_url': 'AMap'});
    Map<String, dynamic> parameters = {
      'key': key,
    };
    if (ip != null) {
      parameters['ip'] = ip;
    }
    if (sig != null) {
      parameters['sig'] = sig;
    }
    dynamic result = await DSDioUtil.getInstance().get(
      '/v3/ip',
      options: options,
      queryParameters: parameters,
    );
    return IpConfigEntity(
      status: result['status'],
      info: result['info'],
      infocode: result['infocode'],
      province: result['province'],
      city: result['city'],
      adcode: result['adcode'],
      rectangle: result['rectangle'],
    );
  }

  @override
  Future<String?> addressToGps(
      {String? key, String? address, String? city, String? sig}) async {
    Options options = Options(headers: {'base_url': 'AMap'});
    Map<String, dynamic> parameters = {
      'key': key,
    };
    if (city != null) {
      parameters['city'] = city;
    }
    if (sig != null) {
      parameters['sig'] = sig;
    }
    parameters['address'] = address;
    dynamic result = await DSDioUtil.getInstance().get(
      '/v3/geocode/geo',
      options: options,
      queryParameters: parameters,
    );
    if (result == null ||
        result['geocodes'] == null ||
        result['geocodes'].length == 0) {
      return null;
    }
    return result['geocodes'][0]['location'];
  }

  @override
  Future geocodeAddress({String? key, String? location}) async {
    Options options = Options(headers: {'base_url': 'AMap'});
    Map<String, dynamic> parameters = {
      'key': key,
      'location':location,
      'extensions':'all',
      'radius':'1000'
    };
    dynamic result = await DSDioUtil.getInstance().get(
      '/v3/geocode/regeo',
      options: options,
      queryParameters:parameters,
    );
    return result;
  }
}
